import matplotlib.pyplot as plt
import matplotlib
import numpy as np
from matplotlib.collections import LineCollection
from matplotlib.colors import Normalize
from matplotlib.cm import ScalarMappable
from config import Config
import plotly.graph_objects as go
import pandas as pd
from matplotlib.colors import Normalize

def gradient_heatmap(data, colormap='viridis', title="Gradient HeatMap", x_label="Data Point Index", save_path=None):
    if Config.plot_backend == 'matplotlib':
        gradient_heatmap_matplotlib(data, colormap=colormap, title=title, x_label=x_label,save_path=save_path)
    elif Config.plot_backend == 'plotly':
        gradient_heatmap_plotly(data, colormap=colormap, title=title, x_label=x_label,save_path=save_path)


def gradient_heatmap_matplotlib(data, colormap='viridis', title="Data Visualization", x_label="testing x",save_path=None):
    fig, ax = plt.subplots()
    
    norm = Normalize(vmin=min(data), vmax=max(data))
    cmap_name = colormap + '_r'
    cmap = matplotlib.colormaps[cmap_name]

    dpi = fig.get_dpi()
    fig_width_in_inches = fig.get_figwidth()
    total_points = len(data)
    linewidth = (fig_width_in_inches * dpi) / total_points

    lines = []
    for i, value in enumerate(data):
        color = cmap(norm(value))
        line = [(i, 0), (i, 1)]
        lines.append((line, color))

    line_segments = LineCollection([line for line, color in lines], colors=[color for line, color in lines], linewidths=(linewidth,))
    ax.add_collection(line_segments)

    sm = ScalarMappable(cmap=cmap, norm=norm)
    sm.set_array([])
    fig.colorbar(sm, ax=ax)

    ax.set_xlim(0, len(data))
    ax.set_ylim(0, 1)
    ax.set_yticks([])
    ax.set_xlabel(x_label)
    ax.set_title(title)
    
    if save_path:
        plt.savefig(save_path)
    else:
        plt.show()


def gradient_heatmap_plotly(data, colormap='Viridis', title="Data Visualization", x_label="Data Point Index", save_path=None):
    x_data = list(range(len(data)))
    y_data = [1] * len(data)
    hover_texts = [f"Value: {value:.2f}" for value in data]
    
    colorscale_name = colormap + "_r"
    
    fig = go.Figure(data=go.Bar(
        x=x_data,
        y=y_data,
        text=hover_texts,
        hoverinfo="text+x",
        marker=dict(
            color=data,
            colorscale=colorscale_name,
            # colorbar=dict(title=y_label)
        ),
        width=np.full(len(data), 1.05)
    ))

    fig.update_layout(
        xaxis=dict(
            title=x_label,
            showticklabels=True,
        ),
        yaxis=dict(
            showticklabels=False,
            title="",
        ),
        title=title,
        paper_bgcolor="white",
        plot_bgcolor="white",
        margin=dict(l=40, r=40, t=40, b=40),
    )

    if save_path:
        fig.write_html(save_path)
    else:
        fig.show()


def month_year_heatmap(df, title="", colormap='viridis', save_path=None):
    if Config.plot_backend == 'matplotlib':
        month_year_heatmap_matplotlib(df=df, title=title, colormap=colormap, save_path=save_path)
    elif Config.plot_backend == 'plotly':
        month_year_heatmap_plotly(df=df, title=title, colormap=colormap, save_path=save_path)

def month_year_heatmap_matplotlib(df, title='Month-Year Heatmap', colormap='viridis', save_path=None):
    norm = Normalize(vmin=df.min().min(), vmax=df.max().max())
    cmap = matplotlib.colormaps[colormap + "_r"]

    fig, ax = plt.subplots()
    cax = ax.imshow(df.values, cmap=cmap, norm=norm, aspect='auto')

    ax.set_xticks(np.arange(len(df.columns)))
    ax.set_yticks(np.arange(len(df.index)))
    ax.set_xticklabels(df.columns)
    ax.set_yticklabels(df.index)
    
    plt.setp(ax.get_xticklabels(), rotation=45, ha="right", rotation_mode="anchor")
    cbar = plt.colorbar(cax, ax=ax)
    cbar.ax.set_ylabel('Magnitude', rotation=-90, va="bottom")
    ax.set_title(title)

    if save_path:
        plt.savefig(save_path)
    else:
        plt.show()

def month_year_heatmap_plotly(df, title='Month-Year Heatmap', colormap='Viridis', save_path=None):

    fig = go.Figure(data=go.Heatmap(
        z=df.values,
        x=df.columns,
        y=df.index,
        colorscale=colormap + "_r",
        colorbar=dict(title='Magnitude'),
    ))

    fig.update_layout(
        title=title,
        xaxis=dict(title='Year', type='category'),
        yaxis=dict(title='Month'),
    )
    if save_path:
        fig.write_html(save_path)
    else:
        fig.show()





if __name__ == "__main__":
    li = np.concatenate([np.random.randint(0, 20, size=400), np.random.randint(20, 30, size=400), np.random.randint(30, 100, size=400)])
    # print(li)
    gradient_heatmap(li)
    # data = np.random.rand(100)

    # data = pd.DataFrame({
    #     'Category 1': np.random.rand(10),
    #     'Category 2': np.random.rand(10),
    #     'Category 3': np.random.rand(10),
    # }, index=pd.date_range(start='2021-01-01', periods=10, freq='D'))
    
    np.random.seed(0)
    data = np.random.rand(12, 31)  #12 months, 5 years of data

    years = [2010,2011,2012,2013,2014,2015,2016, 2017, 2018, 2019, 2020,2021,2022,2023,2024,2025,2026,2027,2028,2029,2030,2031,2032,2033,2034,2035,2036,2037,2038,2039,2040]
    months = [
        "January", "February", "March", "April", "May", "June",
        "July", "August", "September", "October", "November", "December"
    ]

    df = pd.DataFrame(data, index=months, columns=years)
    # month_year_heatmap(df, title='Month-Year Heatmap Test', colormap="magma")


from .functions import barplot, lineplot, scatterplot, jointplot, swarmplot
from .colormap import plot_colormap_gradient, plot_colormap_in_rgb_space, create_custom_colormap, delta_e, delta_e_lightness, convert_colormap_for_colorblindness, compare_colormaps
from .heatmap import gradient_heatmap, month_year_heatmap
from .config import Config, set_plot_backend

__version__ = '0.2.5'
__author__ = 'Molly Nelson'

print("Package Landborn 0.2.5 Successfully Imported")


import numpy as np
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import pandas as pd
import matplotlib.patches as patches
from matplotlib.path import Path
import plotly.graph_objects as go
from .config import Config

#scatterplot

marker_conversion_map = {
    'matplotlib_to_plotly': {
        '.': 'circle',
        'o': 'circle',
        'v': 'triangle-down',
        '^': 'triangle-up',
        '<': 'triangle-left',
        '>': 'triangle-right',
        '1': 'triangle-up',
        '2': 'triangle-down',
        '3': 'triangle-left',
        '4': 'triangle-right',
        's': 'square',
        'p': 'pentagon',
        '*': 'star',
        'h': 'hexagon',
        'H': 'hexagon',
        '+': 'cross',
        'x': 'x',
        'D': 'diamond',
        'd': 'diamond',
        '|': 'line-ns',
        '_': 'line-ew',
    },
    'plotly_to_matplotlib': {
        'circle': 'o',
        'square': 's',
        'diamond': 'D',
        'cross': '+',
        'x': 'x',
        'triangle-up': '^',
        'triangle-down': 'v',
        'triangle-left': '<',
        'triangle-right': '>',
        'pentagon': 'p',
        'hexagon': 'h',
        'hexagon2': 'H',
        'star': '*',
        'line-ns': '|',
        'line-ew': '_',
    }
}

def convert_marker(marker, to='plotly'):
    if to == 'plotly':
        return marker_conversion_map['matplotlib_to_plotly'].get(marker, marker)
    elif to == 'matplotlib':
        return marker_conversion_map['plotly_to_matplotlib'].get(marker, marker)
    else:
        return marker

def scatterplot(df, xvar, yvar, color='#ff0000', colormap='viridis', size=1, marker='.', ax=None, save_path=None):
    if Config.plot_backend == 'matplotlib':
        matplotlib_marker = convert_marker(marker, to='matplotlib')
        return scatterplot_matplotlib(df, xvar, yvar, color=color, colormap=colormap, size=size, marker=matplotlib_marker, ax=ax, save_path=save_path)
    elif Config.plot_backend == 'plotly':
        plotly_marker = convert_marker(marker, to='plotly')
        return scatterplot_plotly(df, xvar, yvar, color=color, size=10*size, marker=plotly_marker, save_path=save_path)



def scatterplot_plotly(df=None, xvar=None, yvar=None, color='blue', colormap='Viridis', size=10, marker='circle', save_path=None):
    xdata = df[xvar] if df is not None and isinstance(xvar, str) else xvar
    ydata = df[yvar] if df is not None and isinstance(yvar, str) else yvar

    fig = go.Figure()

    if df is not None and isinstance(color, str) and color in df.columns:
        fig.add_trace(go.Scatter(x=xdata, y=ydata, mode='markers',
                                 marker=dict(size=size, color=df[color], colorscale=colormap, showscale=True),
                                 name=''))
    else:
        fig.add_trace(go.Scatter(x=xdata, y=ydata, mode='markers',
                                 marker=dict(color=color, size=size, symbol=marker),
                                 name=''))

    xaxis_title = xvar if isinstance(xvar, str) else 'x-axis'
    yaxis_title = yvar if isinstance(yvar, str) else 'y-axis'
    fig.update_layout(title="Scatter Plot",
                      xaxis_title=xaxis_title,
                      yaxis_title=yaxis_title,
                      legend_title="Legend",
                      template="plotly_white")

    if save_path is not None:
        fig.write_html(save_path)
    else:
        fig.show()

    return fig


def scatterplot_matplotlib(df, xvar, yvar, color='k',colormap='viridis', size=1, marker='.', ax=None, save_path=None):
    print('marplotlib')
    if ax is None:
        fig, ax = plt.subplots()
    if isinstance(xvar, str):
        xdata = df[xvar]
    else:
        xdata = xvar
        
    if isinstance(yvar, str):
        ydata = df[yvar]
    else:
        ydata = yvar
    x_data_length = max(xdata) - min(xdata)
    x_patch_length = (x_data_length / len(xdata)) / 5
    print(x_patch_length)
    
    y_data_length = max(ydata) - min(ydata)
    y_patch_length = (y_data_length / len(ydata)) / 5
    print(y_patch_length)
    patches_li = []
    for i, (x, y) in enumerate(zip(xdata, ydata)):
        #creating small square for "point"
        verts = [
        (x - x_patch_length, y - y_patch_length),  #left, bottom
        (x - x_patch_length, y + y_patch_length),  #left, top
        (x + x_patch_length, y + y_patch_length),  #right, top
        (x + x_patch_length, y - y_patch_length),  #right, bottom
        (x, y),  #back to start
        ]

        codes = [
            Path.MOVETO,
            Path.LINETO,
            Path.LINETO,
            Path.LINETO,
            Path.CLOSEPOLY,
        ]
        path = Path(verts, codes)
        patch = patches.PathPatch(path, lw=1, color=color)
        patches_li.append(patch)
    
    for patch in patches_li:
        ax.add_patch(patch)
    ax.set_xlim(min(xdata), max(xdata))
    ax.set_ylim(min(ydata), max(ydata))
    # ax.autoscale()
    ax.legend()
    
    if save_path:
        plt.savefig(save_path)
    else:
        plt.show()
    return ax

#line plot

def lineplot(df, xvar, yvar, color='black', colormap = None, size=6, style='-', marker=None, save_path=None):
    if Config.plot_backend == 'matplotlib':
        print('matplotlib')
        # (df, x=[], y=[], color='k', size=1, style='-', marker=None, ax=None, save_path=None)
        matplotlib_marker = convert_marker(marker, to='matplotlib')
        return lineplot_matplotlib(df, xvar, yvar, color, size, style, marker=matplotlib_marker, save_path=save_path)
    elif Config.plot_backend == 'plotly':
        print('plotly')
        plotly_marker = convert_marker(marker, to='plotly')
        return lineplot_plotly(df, xvar, yvar, color, size=size, style=style, marker=plotly_marker, save_path=save_path)



def lineplot_plotly(df, x=[], y=[], color='black', size=6, style='solid', marker=None, save_path=None):
    line_styles = {
        'solid': None,
        'dash': 'dash',
        'dot': 'dot',
        'dashdot': 'dashdot',
    }
    line_style = line_styles.get(style, None)

    if isinstance(x, str) and df is not None:
        xdata = df[x]
    else:
        xdata = x
    if isinstance(y, str):
        ydata = df[y]
    else:
        ydata = y

    fig = go.Figure(data=go.Scatter(x=xdata, y=ydata, 
                                    mode='lines+markers' if marker else 'lines',
                                    line=dict(color=color, dash=line_style),
                                    marker=dict(color=color, size=size, symbol=marker)))
    
    fig.update_layout(title="Line Plot",xaxis_title=str(x),yaxis_title=str(y),legend_title="Legend",template="plotly_white")

    if save_path:
        fig.write_html(save_path)
    else:
        fig.show()

    return fig



def lineplot_matplotlib(df, x=[], y=[], color='k', size=1, style='-', marker=None, ax=None, save_path=None):
    if ax is None:
        fig, ax = plt.subplots()

    if isinstance(x, str) and df is not None:
        xdata = df[x]
    else:
        xdata = x
    if isinstance(y, str):
        ydata = df[y]
    else:
        ydata = y

    line = mlines.Line2D(xdata, ydata, color=color, linestyle=style, marker=marker, markersize=size)
    ax.add_line(line)
    ax.autoscale()
    
    if save_path:
        plt.savefig(save_path)

    return ax

#barplot

def barplot(df, xvar, yvar, orientation='vertical', color='lightblue', save_path=None, axis=None):
    if Config.plot_backend == 'matplotlib':
        return barplot_matplotlib(df, xvar, yvar, orientation=orientation, color=color, axis=axis, save_path=save_path)
    elif Config.plot_backend == 'plotly':
        return barplot_plotly(df, xvar, yvar, orientation=orientation, color=color, save_path=save_path)

def barplot_plotly(df, xvar, yvar, orientation='vertical', color='lightblue', save_path=None):
    if orientation == 'vertical':
        means = df.groupby(yvar)[xvar].mean()
        sems = df.groupby(yvar)[xvar].sem()
    else:  #orientation == 'horizontal'
        means = df.groupby(yvar)[xvar].mean()
        sems = df.groupby(yvar)[xvar].sem()

    categories = means.index.tolist()
    means = means.tolist()
    sems = sems.tolist()

    if orientation == 'vertical':
        fig = go.Figure(data=go.Bar(
            x=categories,
            y=means,
            marker_color=color,
            error_y=dict(type='data', array=sems, visible=True)  # Add vertical error bars
        ))
    elif orientation == 'horizontal':
        fig = go.Figure(data=go.Bar(
            y=categories,
            x=means,
            marker_color=color,
            orientation='h',
            error_x=dict(type='data', array=sems, visible=True)  # Add horizontal error bars
        ))
    else:
        raise ValueError("Invalid orientation. Choose 'vertical' or 'horizontal'.")

    fig.update_layout(
        title=f"{yvar} by {xvar}",
        xaxis_title=xvar if orientation == 'vertical' else yvar,
        yaxis_title=yvar if orientation == 'vertical' else xvar,
        template="plotly_white"
    )

    if save_path:
        fig.write_html(save_path)
    else:
        fig.show()

    return fig

def barplot_matplotlib(df, xvar, yvar, orientation='vertical', color='lightblue', axis=None, save_path=None):
    if axis is None:
        fig, axis = plt.subplots()

    if orientation == 'vertical':
        if isinstance(xvar, str):
            categories = df[xvar].unique()
            x = np.arange(len(categories))
            heights = df.groupby(xvar)[yvar].mean()
            errors = df.groupby(xvar)[yvar].sem()
        else:
            x = xvar
            heights = x.mean()
            errors = x.sem()
        bars = []
        for i, (category, height) in enumerate(zip(categories, heights)):
            error_line = mlines.Line2D([x[i], x[i]], [height-(errors[category]/2), height+(errors[category]/2)], color='black')
            error_line_top = mlines.Line2D([x[i] - 0.1, x[i] + 0.1], [height+(errors[category]/2), height+(errors[category]/2)], color='black')
            error_line_bottom = mlines.Line2D([x[i] - 0.1, x[i] + 0.1], [height-(errors[category]/2), height-(errors[category]/2)], color='black')
            rect = patches.Rectangle((x[i] - 0.4, 0), 0.8, height,facecolor=color, edgecolor='black')
            bars.append(rect)
            axis.add_patch(rect)
            axis.add_line(error_line)
            axis.add_line(error_line_top)
            axis.add_line(error_line_bottom)

        axis.set_xticks(x)
        axis.set_xticklabels(categories)
        axis.set_xlabel(xvar)
        axis.set_ylabel(yvar)
        axis.autoscale()
        axis.set_ylim(bottom=0)

    elif orientation == 'horizontal':
        temp = yvar
        yvar = xvar
        xvar = temp
        categories = df[yvar].unique()
        y = np.arange(len(categories))
        heights = df.groupby(yvar)[xvar].mean()
        errors = df.groupby(yvar)[xvar].sem()

        bars = []
        for i, (category, height) in enumerate(zip(categories, heights)):
            error_line = mlines.Line2D([height-(errors[category]/2), height+(errors[category]/2)], [y[i], y[i]],color='black')
            error_line_top = mlines.Line2D([height+(errors[category]/2), height+(errors[category]/2)], [y[i] - 0.1, y[i] + 0.1], color='black')
            error_line_bottom = mlines.Line2D([height-(errors[category]/2), height-(errors[category]/2)], [y[i] - 0.1, y[i] + 0.1], color='black')
            rect = patches.Rectangle((0, y[i] - 0.4), height, 0.8,facecolor=color, edgecolor='black')
            bars.append(rect)
            axis.add_patch(rect)
            axis.add_line(error_line)
            axis.add_line(error_line_top)
            axis.add_line(error_line_bottom)

        axis.set_yticks(y)
        axis.set_yticklabels(categories)
        axis.set_ylabel(yvar)
        axis.set_xlabel(xvar)
        axis.autoscale()
        axis.set_xlim(0)
    if save_path:
        plt.savefig(save_path)
    else:
        plt.show()

    return axis

def jointplot(x, y, ax=None, color='black', title='Joint Plot', save_path=None):
    fig, axs = plt.subplots(2, 1, figsize=(10, 8), sharex=True)

    #top line plot
    axs[0].set_title(title)
    axs[0].tick_params(axis='x',which='both', bottom=False,top=False,labelbottom=False)
    axs[0].spines['bottom'].set_visible(False)
    axs[0].spines['right'].set_visible(False)
    axs[0].spines['top'].set_visible(False)
    lineplot(None,x,y, color=color, ax=axs[0])

    #bottom scatterplot
    axs[1].spines['top'].set_visible(False)
    axs[1].spines['right'].set_visible(False)
    axs[1].set_xlabel('X axis')
    scatterplot(None, x,y, color=color, ax=axs[1])
    
    if save_path:
        plt.savefig(save_path)


def collides(x1,y1,x2,y2, r):
    return np.sqrt((x1-x2)**2 + (y1-y2)**2) < 2 * r

def check_collisions(candidate_ys, collisions, x, r):
    candidate_ys = sorted(candidate_ys, key=np.abs)
    for y in candidate_ys:
        has_collision = False
        for (x_prime, y_prime) in collisions:
            if collides(x, y, x_prime, y_prime, r): 
                has_collision = True
                break
        if not has_collision:
            return y

def swarmplot_inner(X, C, y_offset = 0, r=0.05, ax=None):
    Y = np.zeros_like(X)
    if ax == None:
        fig, ax = plt.subplots()
    for i in range(len(X)):
        collisions = []
        for j in range(i-1,0,-1):
            if X[i] - X[j] > 2 * r:
                break
            else:
                collisions.append((X[j], Y[j]))

        candidate_ys = [0]
        for (x_prime, y_prime) in collisions:
            candidate_ys.append(y_prime + np.sqrt(np.abs((2 * r) ** 2 - (X[i] - x_prime) ** 2)))
            candidate_ys.append(y_prime - np.sqrt(np.abs((2 * r) ** 2 - (X[i] - x_prime) ** 2)))
        
        Y[i] = check_collisions(candidate_ys, collisions, X[i], r)

    scatterplot(None, X, y_offset + Y, color=C, ax=ax)
    
def swarmplot(df, categorical_data, numerical_data, r=0.5, ax=None, save_path=None):
    if ax == None:
        fig, ax = plt.subplots()
    categories = df[categorical_data].unique()
    counter = 0
    colors = ['red', 'green', 'blue']
    for category in categories:
        passed_data = list(df.loc[df[categorical_data] == category][numerical_data])
        swarmplot_inner(passed_data, colors[counter], y_offset=counter,r=r, ax=ax)
        counter += 1
    ax.set_ylabel(categorical_data)
    ax.set_xlabel(numerical_data)
    plt.gca().set_aspect('equal', adjustable='box')
    if save_path:
        plt.savefig(save_path)

if __name__ == '__main__':
    np.random.seed(120)
    data = pd.DataFrame({
    'Category': ['A']*80 + ['B']*80 + ['C']*80,
    'Value': np.concatenate([np.random.randint(0, 20, size=80), np.random.randint(20, 30, size=80), np.random.randint(30, 50, size=80)])
    })
    x = [1, 2, 3, 4, 5]
    y = [10, 15, 7, 10, 5]

    # lineplot(None,x,y, color='b',save_path="matplottest.png")
    set_plot_backend('plotly')
    scatterplot(None, x, y, color="red", save_path="scatter.html")
    # swarmplot(data, 'Category', 'Value', r=0.8, save_path='test_swarmplot_confirmed.png')
    # Config.set_plot_backend('plotly')
    # lineplot(data,'Category', 'Value', color='purple')


import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import matplotlib.patches as mpatches
import numpy as np
import pandas as pd
import plotly.graph_objects as go
from config import Config

marker_conversion_map = {
    'matplotlib_to_plotly': {
        '.': 'circle',
        'o': 'circle',
        'v': 'triangle-down',
        '^': 'triangle-up',
        '<': 'triangle-left',
        '>': 'triangle-right',
        '1': 'triangle-up',
        '2': 'triangle-down',
        '3': 'triangle-left',
        '4': 'triangle-right',
        's': 'square',
        'p': 'pentagon',
        '*': 'star',
        'h': 'hexagon',
        'H': 'hexagon',
        '+': 'cross',
        'x': 'x',
        'D': 'diamond',
        'd': 'diamond',
        '|': 'line-ns',
        '_': 'line-ew',
    },
    'plotly_to_matplotlib': {
        'circle': 'o',
        'square': 's',
        'diamond': 'D',
        'cross': '+',
        'x': 'x',
        'triangle-up': '^',
        'triangle-down': 'v',
        'triangle-left': '<',
        'triangle-right': '>',
        'pentagon': 'p',
        'hexagon': 'h',
        'hexagon2': 'H',
        'star': '*',
        'line-ns': '|',
        'line-ew': '_',
        # Note: Some Plotly markers don't have direct Matplotlib equivalents
    }
}

def convert_marker(marker, to='plotly'):
    """
    Converts a marker style from Matplotlib to Plotly or vice versa.
    
    Parameters:
    - marker: The marker style to convert.
    - to: The target library ('plotly' or 'matplotlib').
    
    Returns:
    - The converted marker style, if a conversion is found; otherwise, the original marker.
    """
    if to == 'plotly':
        return marker_conversion_map['matplotlib_to_plotly'].get(marker, marker)
    elif to == 'matplotlib':
        return marker_conversion_map['plotly_to_matplotlib'].get(marker, marker)
    else:
        return marker

def lineplot(df, xvar, yvar, color='black', colormap = None, size=6, style='-', marker=None, save_path=None):
    if Config.plot_backend == 'matplotlib':
        print('matplotlib')
        # (df, x=[], y=[], color='k', size=1, style='-', marker=None, ax=None, save_path=None)
        matplotlib_marker = convert_marker(marker, to='matplotlib')
        return lineplot_matplotlib(df, xvar, yvar, color, size, style, marker=matplotlib_marker, save_path=save_path)
    elif Config.plot_backend == 'plotly':
        print('plotly')
        plotly_marker = convert_marker(marker, to='plotly')
        return lineplot_plotly(df, xvar, yvar, color, size=size, style=style, marker=plotly_marker, save_path=save_path)



def lineplot_plotly(df, x=[], y=[], color='black', size=6, style='solid', marker=None, save_path=None):
    # Define the line style
    line_styles = {
        'solid': None,
        'dash': 'dash',
        'dot': 'dot',
        'dashdot': 'dashdot',
    }
    line_style = line_styles.get(style, None)

    # Determine x and y data
    if isinstance(x, str) and df is not None:
        xdata = df[x]
    else:
        xdata = x
    if isinstance(y, str):
        ydata = df[y]
    else:
        ydata = y

    # Create the Plotly figure
    fig = go.Figure(data=go.Scatter(x=xdata, y=ydata, 
                                    mode='lines+markers' if marker else 'lines',
                                    line=dict(color=color, dash=line_style),
                                    marker=dict(color=color, size=size, symbol=marker)))
    
    # Optional: Customize the layout
    fig.update_layout(title="Line Plot",xaxis_title=str(x),yaxis_title=str(y),legend_title="Legend",template="plotly_white")

    # Save the plot if a save_path is provided
    if save_path:
        fig.write_html(save_path)
    else:
        fig.show()

    return fig



def lineplot_matplotlib(df, x=[], y=[], color='k', size=1, style='-', marker=None, ax=None, save_path=None):
    if ax is None:
        fig, ax = plt.subplots()

    if isinstance(x, str) and df is not None:
        xdata = df[x]
    else:
        xdata = x
    if isinstance(y, str):
        ydata = df[y]
    else:
        ydata = y

    line = mlines.Line2D(xdata, ydata, color=color, linestyle=style, marker=marker, markersize=size)
    ax.add_line(line)
    ax.autoscale()
    
    if save_path:
        plt.savefig(save_path)

    return ax

if __name__ == "__main__":
    x = [1, 2, 3, 4, 5]
    y = [10, 15, 7, 10, 5]
    data = pd.DataFrame({
    'Category': np.concatenate([np.random.randint(0, 20, size=80), np.random.randint(20, 30, size=80), np.random.randint(30, 50, size=80)]),
    'Value': np.concatenate([np.random.randint(0, 20, size=80), np.random.randint(20, 30, size=80), np.random.randint(30, 50, size=80)])
    })
    plt = lineplot(data, 'Category', 'Value', color='yellow',save_path="test_lineplot.html")


import matplotlib.patches as patches
from matplotlib.path import Path
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import plotly.graph_objects as go
from config import Config

marker_conversion_map = {
    'matplotlib_to_plotly': {
        '.': 'circle',
        'o': 'circle',
        'v': 'triangle-down',
        '^': 'triangle-up',
        '<': 'triangle-left',
        '>': 'triangle-right',
        '1': 'triangle-up',
        '2': 'triangle-down',
        '3': 'triangle-left',
        '4': 'triangle-right',
        's': 'square',
        'p': 'pentagon',
        '*': 'star',
        'h': 'hexagon',
        'H': 'hexagon',
        '+': 'cross',
        'x': 'x',
        'D': 'diamond',
        'd': 'diamond',
        '|': 'line-ns',
        '_': 'line-ew',
    },
    'plotly_to_matplotlib': {
        'circle': 'o',
        'square': 's',
        'diamond': 'D',
        'cross': '+',
        'x': 'x',
        'triangle-up': '^',
        'triangle-down': 'v',
        'triangle-left': '<',
        'triangle-right': '>',
        'pentagon': 'p',
        'hexagon': 'h',
        'hexagon2': 'H',
        'star': '*',
        'line-ns': '|',
        'line-ew': '_',
    }
}

def convert_marker(marker, to='plotly'):
    if to == 'plotly':
        return marker_conversion_map['matplotlib_to_plotly'].get(marker, marker)
    elif to == 'matplotlib':
        return marker_conversion_map['plotly_to_matplotlib'].get(marker, marker)
    else:
        return marker

def scatterplot(df, xvar, yvar, color='#ff0000', colormap='viridis', size=1, marker='.', ax=None, save_path=None):
    if Config.plot_backend == 'matplotlib':
        matplotlib_marker = convert_marker(marker, to='matplotlib')
        return scatterplot_matplotlib(df, xvar, yvar, color=color, colormap=colormap, size=size, marker=matplotlib_marker, ax=ax, save_path=save_path)
    elif Config.plot_backend == 'plotly':
        plotly_marker = convert_marker(marker, to='plotly')
        return scatterplot_plotly(df, xvar, yvar, color=color, size=10*size, marker=plotly_marker, save_path=save_path)  # Adjust size as an example



def scatterplot_plotly(df=None, xvar=None, yvar=None, color='blue', colormap='Viridis', size=10, marker='circle', save_path=None):
    # Handle xvar and yvar when they are column names or lists directly
    xdata = df[xvar] if df is not None and isinstance(xvar, str) else xvar
    ydata = df[yvar] if df is not None and isinstance(yvar, str) else yvar

    # Initialize the figure
    fig = go.Figure()

    # Check if color needs to be applied as a colormap from the DataFrame
    if df is not None and isinstance(color, str) and color in df.columns:
        fig.add_trace(go.Scatter(x=xdata, y=ydata, mode='markers',
                                 marker=dict(size=size, color=df[color], colorscale=colormap, showscale=True),
                                 name=''))
    else:
        # Apply a single color to all markers if color is not a column name
        fig.add_trace(go.Scatter(x=xdata, y=ydata, mode='markers',
                                 marker=dict(color=color, size=size, symbol=marker),
                                 name=''))

    # Optional: Customize layout (title, axis labels, etc.)
    xaxis_title = xvar if isinstance(xvar, str) else 'x-axis'
    yaxis_title = yvar if isinstance(yvar, str) else 'y-axis'
    fig.update_layout(title="Scatter Plot",
                      xaxis_title=xaxis_title,
                      yaxis_title=yaxis_title,
                      legend_title="Legend",
                      template="plotly_white")

    # Save the plot if a save_path is provided
    if save_path is not None:
        fig.write_html(save_path)
    else:
        fig.show()

    return fig


def scatterplot_matplotlib(df, xvar, yvar, color='k',colormap='viridis', size=1, marker='.', ax=None, save_path=None):
    print('marplotlib')
    if ax is None:
        fig, ax = plt.subplots()
    if isinstance(xvar, str):
        xdata = df[xvar]
    else:
        xdata = xvar
        
    if isinstance(yvar, str):
        ydata = df[yvar]
    else:
        ydata = yvar
    x_data_length = max(xdata) - min(xdata)
    x_patch_length = (x_data_length / len(xdata)) / 5
    print(x_patch_length)
    
    y_data_length = max(ydata) - min(ydata)
    y_patch_length = (y_data_length / len(ydata)) / 5
    print(y_patch_length)
    patches_li = []
    for i, (x, y) in enumerate(zip(xdata, ydata)):
        #creating small square for "point"
        verts = [
        (x - x_patch_length, y - y_patch_length),  #left, bottom
        (x - x_patch_length, y + y_patch_length),  #left, top
        (x + x_patch_length, y + y_patch_length),  #right, top
        (x + x_patch_length, y - y_patch_length),  #right, bottom
        (x, y),  #back to start
        ]

        codes = [
            Path.MOVETO,
            Path.LINETO,
            Path.LINETO,
            Path.LINETO,
            Path.CLOSEPOLY,
        ]
        path = Path(verts, codes)
        patch = patches.PathPatch(path, lw=1, color=color)
        patches_li.append(patch)
    
    for patch in patches_li:
        ax.add_patch(patch)
    ax.set_xlim(min(xdata), max(xdata))
    ax.set_ylim(min(ydata), max(ydata))
    # ax.autoscale()
    ax.legend()
    
    if save_path:
        plt.savefig(save_path)
    else:
        plt.show()
    return ax

if __name__ == "__main__":
    x = [1, 2, 3, 4, 5]
    y = [10, 15, 7, 10, 5]
    data = pd.DataFrame({
    'Category': np.concatenate([np.random.randint(0, 20, size=80), np.random.randint(20, 30, size=80), np.random.randint(30, 50, size=80)]),
    'Value': np.concatenate([np.random.randint(0, 20, size=80), np.random.randint(20, 30, size=80), np.random.randint(30, 50, size=80)])
    })
    # Config.set_plot_backend('plotly')
    plt = scatterplot(None,x, y, color='purple',marker="circle", save_path="test_scatterplot.png")


# In config.py

class Config:
    # Default to 'matplotlib'
    plot_backend = 'plotly'

def set_plot_backend(backend_name):
    if backend_name not in ['matplotlib', 'plotly']:
        raise ValueError("Backend not supported. Choose 'matplotlib' or 'plotly'.")
    else:
        Config.plot_backend = backend_name
